# Rocky Linux 9 Server
# ---
# Packer Template to create an Rocky Linux 9 Server on Proxmox

# Variable Definitions
variable "proxmox_api_url" {
    type = string
}

variable "proxmox_api_token_id" {
    type = string
}

variable "proxmox_api_token_secret" {
    type = string
    sensitive = true
}

source "proxmox" "rocky-linux-9" {

    # Proxmox Connection Settings
    proxmox_url = "${var.proxmox_api_url}"
    username = "${var.proxmox_api_token_id}"
    token = "${var.proxmox_api_token_secret}"
    # (Optional) Skip TLS Verification
    insecure_skip_tls_verify = true

    # VM General Settings
    node = "boba"
    vm_id = "4500"
    vm_name = "rocky-linux-9"
    template_description = "Rocky Linux 9 Server Image"

    # VM OS Settings
    # (Option 1) Local ISO File
    # iso_file = "local:iso/ubuntu-20.04.2-live-server-amd64.iso"
    # - or -
    # (Option 2) Download ISO
    # iso_url = "https://releases.ubuntu.com/20.04/ubuntu-20.04.3-live-server-amd64.iso"
    # iso_checksum = "f8e3086f3cea0fb3fefb29937ab5ed9d19e767079633960ccb50e76153effc98"
    iso_file = "data:iso/rocky-9.2-x86_64-dvd.iso"
    iso_storage_pool = "data"
    unmount_iso = true

    # VM System Settings
    qemu_agent = true

    # VM Hard Disk Settings
    scsi_controller = "virtio-scsi-pci"

    disks {
        disk_size = "40G"
        format = "qcow2"
        storage_pool = "fast"
        storage_pool_type = "storage"
        type = "virtio"
    }

    # VM CPU Settings
    cores = "2"

    # VM Memory Settings
    memory = "2048"

    # VM Network Settings
    network_adapters {
        model = "virtio"
        bridge = "vmbr0"
        firewall = "false"
    }

    # VM Cloud-Init Settings
    cloud_init = true
    cloud_init_storage_pool = "fast"

    # PACKER Boot Commands
    boot_command = ["<tab> text inst.ks=http://{{ .HTTPIP }}:{{ .HTTPPort }}/files/inst.ks<enter><wait>"]
    boot_wait = "10s"

    # PACKER Autoinstall Settings
    http_directory = "http"
    # (Optional) Bind IP Address and Port
    # http_bind_address = "0.0.0.0"
    # http_port_min = 8802
    # http_port_max = 8802

    ssh_username = "cnorris"

    # (Option 1) Add your Password here
    # ssh_password = "your-password"
    # - or -
    # (Option 2) Add your Private SSH KEY file here
    ssh_private_key_file = "~/.ssh/id_rsa"

    # Raise the timeout, when installation takes longer
    ssh_timeout = "20m"
}

# Build Definition to create the VM Template
build {

    name = "rocky-linux-9"
    sources = ["source.proxmox.rocky-linux-9"]

    # Provisioning the VM Template for Cloud-Init Integration in Proxmox #1
    provisioner "shell" {
        inline = [
            "echo "Updating system...",
            "dnf -y update",
            "echo "Installing python and ansible...",
            "dnf -y install python3",
            "dnf -y install python3-pip",
            "pip3 install ansible",
            "echo "Installing cloud-init...",
            "dnf -y install cloud-init",
            "echo "manual_cache_clean: True" > /etc/cloud/cloud.cfg.d/99-manual.cfg"
        ]
    }

    provisioner "shell" {
        inline = [
            "dnf install -y cloud-init qemu-guest-agent cloud-utils-growpart gdisk", "systemctl enable qemu-guest-agent",
            "shred -u /etc/ssh/*_key /etc/ssh/*_key.pub",
            "rm -f /var/run/utmp",
            ">/var/log/lastlog",
            ">/var/log/wtmp",
            ">/var/log/btmp",
            "rm -rf /tmp/* /var/tmp/*",
            "unset HISTFILE; rm -rf /home/*/.*history /root/.*history",
            "rm -f /root/*ks",
            "passwd -d root",
            "passwd -l root",
            "rm -f /etc/ssh/ssh_config.d/allow-root-ssh.conf"
        ]
    }

    # Provisioning the VM Template for Cloud-Init Integration in Proxmox #2
    provisioner "file" {
        source = "files/99-pve.cfg"
        destination = "/tmp/99-pve.cfg"
    }

    # Provisioning the VM Template for Cloud-Init Integration in Proxmox #3
    provisioner "shell" {
        inline = [ "sudo cp /tmp/99-pve.cfg /etc/cloud/cloud.cfg.d/99-pve.cfg" ]
    }

    # Add additional provisioning scripts here
    # ...
}